# NOTE: Derived from blib/lib/Net/UKDomain/Nominet/Automaton.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package Net::UKDomain::Nominet::Automaton;

#line 458 "blib/lib/Net/UKDomain/Nominet/Automaton.pm (autosplit into blib/lib/auto/Net/UKDomain/Nominet/Automaton/operation.al)"
sub operation {
	# http://www.nominet.org.uk/registrars/systems/auto/operations/
	my $idetails = shift;
	#use Data::Dumper; print Dumper(\$idetails);
	
	if (!$idetails) {
		$errstr = "Details are required";
		return;
	}
	
	my $operation = uc($idetails->{'operation'});
	
	if ($operation && $operation !~ m/^$operationsi$/i) {
		$errstr = "Unrecognised operation";
		return;
	}
	
	# Denied operation
	if ($operation eq 'BULK') {
		$errstr = "BULK operation is not supported at this time";
		return;
	}
	
	# We need to map the depreciated fields onto the synonym or drop them
	my @depreciated_fields = keys %depreciated;
	foreach my $field (@depreciated_fields) {
		if ($idetails->{$field}) { 
			$idetails->{$depreciated{$field}} = $idetails->{$field};
			delete $idetails->{$field};
		}
	}

	# Denied fields
	if ($operation ne 'REQUEST' && $idetails->{'account-name'}) {
		$errstr = "The account name cannot be modified";
		return;
	}
	if ($operation eq 'MODIFY' && $idetails->{'co-no'} && $idetails->{'key'} =~ m/\.sch\.uk$/i) {
		$errstr = "The 'co-no' field cannot be modified for a 'sch.uk' domain name";
		return;
	}
	
	if ($operation =~ /QUERY|MODIFY/) {
		if (!($idetails->{'key'} || $idetails->{'dns-id'} || $idetails->{'account-id'} || $idetails->{'contact-id'})) {
			$errstr = "Missing field 'key' or 'dns-id' or 'account-id' or 'contact-id' for operation";
			return;
		}
	}
	
	if ($operation =~ /LIST/) {
		if (!($idetails->{'month'} || $idetails->{'expiry'})) {
			$errstr = "Missing field 'month' or 'expiry' for operation";
			return;
		}
	}
	
	if ($operation =~ /RECEIVE/) {
		if (!($idetails->{'accept'} || $idetails->{'reject'})) {
			$errstr = "Missing field 'accept' or 'reject' for operation";
			return;
		}
	}

	if ($idetails->{'type'} && $idetails->{'country'} && $idetails->{'country'} !~ /GB/i) {
		if ($idetails{'type'} eq 'IND') { $idetails->{'type'}='FIND'; }
		if ($idetails{'type'} eq 'OTHER') { $idetails->{'type'}='FOTHER'; }
	}
	
	# IND and FIND are special
	if ($idetails->{'type'} && $idetails->{'type'} !~ /IND|FIND/i) {
		if ($idetails->{'opt-out'}) { $idetails->{'opt-out'} = 'n'; }
	}
	if ($idetails->{'type'} && $idetails->{'type'} =~ /IND|FIND/i) {
		if ($idetails->{'co-no'}) { $idetails->{'co-no'} = undef; }
	}

	# Check mandatory fields
	if (!&is_mandatory($idetails)) { return; } # $errstr is set in the subroutine

	# Let's check the tag is valid if we specify one
	if ($idetails->{'registrar-tag'} && !&valid_tag($idetails->{'registrar-tag'})) {
		# $errstr will be set in the subroutine
		return;
	}

	# We won't validate the domain, we expect you to do that before it enters the hash

	my $msg = &build_msg($idetails);
	if (!$msg) { return; } # $errstr is set in the subroutine

	my $sign = &sign($msg);
	if (!$sign) { return; } # $errstr is set in the subroutine
	
	# This corrects the subject (must appear after $sign is set)
	if ($operation eq 'QUERY' || $operation eq 'MODIFY') {
		if ($idetails->{'contact-id'}) { $operation = 'contact '.$operation; }
		elsif ($idetails->{'dns-id'}) { $operation = 'nameserver '.$operation; }
		elsif ($idetails->{'account-id'}) { $operation = 'account '.$operation; }
	}
	
	# Ensure operation is correct
	my $to = &send_to($operation,$idetails);

	my $subject = $self{tag}." ".$operation;

	if (!&send_mail($to, $subject, $sign)) {
		if (!$errstr) { $errstr = "Unable to send email request"; }
		return;
	}
	return 1;
}

# end of Net::UKDomain::Nominet::Automaton::operation
1;
